###########################################################################
#
# Copyright(c) 2005 Broadcom Corporation, all rights reserved
# Proprietary and Confidential Information.
#
# This source file is the property of Broadcom Corporation, and
# may not be copied or distributed in any isomorphic form without 
# the prior written consent of Broadcom Corporation. 
#
# Name:		    $Source: /projects/svw_cvs/MediaPC/decoder/apps/testapps/tcl/scripts/bcdiff.tcl,v $
#
# Description:	    Contains implementation of "diff" utility for
#			    decplay command's output CRC files.
#
# Dependency:	    Tcl 8.4.x, Tk 8.4.x, BWidget 1.7
#
# Revision:	    $Revision: #1 $
# Last Modified:    $Date: 2010/02/22 $
#
###########################################################################

# #########################################################################
# 1) First check the continuity of the picture numbers. 
#     a) Zero is not a valid picture number.
#     b) Picture numbers start with 3.
#     c) Picture number 0 before picture number 3 are blank and can be ignored.
#     d) Picture numbers should increment sequentially. Otherwise we have a frame drop.
#     e) If there exists a zero between two picture numbers then there is a PIB drop. 
#        E.g: (7, 0, 9): We got frame 8 but did not receive PIB for frame 8.
# 
# 2) Ignore repeated picture numbers in the middle... But keep the data for result purpose.
# 
# 3) Given the same clip, CRC for a particular picture number (X) should always match. 
#    Otherwise flag it as a frame data miscompare. 
#    So the script need to match the picture number first and then compare the CRC.
# 
# 4) If the total frame count (last picture number at eof) does not match, 
#    then we have a EOS detection issue..
# 
# 5) Ignore repeated or extra frames at the end.. 
# 
# 6) Result dumping:
#   
#    a) Header with files compared, input clip, output file etc..
#    b) Frame drop count
#    c) PIB drop count
#    d) Frame data mismatch count.
#    e) EOS error case (number of extra/less frames at the end)
#    f) Repeated picture count (start/middle, end) 
# ########################################################################

##########################################################################
#
# Command Usage: bcdiff <out_fmt> <file1> <file2>
#   Where, 
#      out_fmt : "-Progressive" or "-Interlaced"
#
# Example: bcdiff -Progressive Zathura1.crc Zathura2.crc
#
##########################################################################

# Specify any dependency on other packages
package require Tcl

# Specify the package this module provides
package provide Bcdiff 1.0

namespace eval Bcdiff {
    set diff_cmd {bcdiff}

    # Export diff command to global scope
    foreach cmd $diff_cmd {
        interp alias {} $cmd {} "Bcdiff::$cmd"
    }
}

# TCL Diff script for decplay output CRC files
proc Bcdiff::bcdiff {outfmt file1 file2} {

	set frmNolist1   {}
	set crcvallist1  {}
	set frmNolist2   {}
	set crcvallist2  {}
	set result1      {}
	set result2      {}


	# Extract Frame no. and CRC value list from both the input files.
	bcextract $file1 frmNolist1 crcvallist1
	bcextract $file2 frmNolist2 crcvallist2

	# Parse the frame no. list for errors
	set result1 [bcChkFrmno $outfmt $frmNolist1]
	set result2 [bcChkFrmno $outfmt $frmNolist2]

  
	set cmpres [bcCmpCRCVal $outfmt $frmNolist1 $crcvallist1 $frmNolist2 $crcvallist2]

	bcOutputResult $file1 $result1 $file2 $result2 $cmpres

	return [llength $cmpres]
}

# Extract frame no. and crc value from the input file
proc Bcdiff::bcextract {file idxlist vallist} {

	upvar 1 $idxlist il
	upvar 1 $vallist vl

	# Open the files
	set fh [open $file r]

	# Read the complete file and extract only the frame no and crc value list
	set filelist [split [read $fh] "\n"]
	set filelistlen [llength $filelist]
	set filelist [lrange $filelist 1 $filelistlen]
	set filelist [lrange $filelist [expr [lsearch $filelist "*=="]+1] $filelistlen]
	set filelist [lrange $filelist 0 [expr [lsearch -exact $filelist {}]-1]]

	# Separate frame no. and crc vale from the file list
	foreach item $filelist {
		set fieldlist [split $item ":"]
		set idx [lindex $fieldlist 0]
		set val [string trimleft [lindex $fieldlist 1]]
		lappend il $idx
		lappend vl $val
	}

	# close the file
	close $fh
}

# Check for PIB misses and frame drops
proc Bcdiff::bcChkFrmno {outfmt frmNoList} {
	set start 		0
	set currFrmNo 	0
	set frmDropCnt 	0
	set frmRepCnt 	0
	set pibMissCnt 	0
	set frmseq		0
	set prevPicNo	0
	set result 		{}

	foreach item $frmNoList {

		# Ignoring the earlier zero frame numbers
		if {$item != 0 && $start != 0} {	

			if {[expr ![string compare $outfmt "-Interlaced"]]} {
				if {$frmseq == 0 && $prevPicNo == 0} {
					set frmseq 1
				} else {
					#puts "i$item"
					incr frmDropCnt
				}
			}


			# Repeated frame condition
			if {$item != $currFrmNo} {

				# Continue checking the frame nos.
				set currFrmNo [expr $currFrmNo+1]
				if {$item != $currFrmNo} {

					# Frame drop
					incr frmDropCnt [expr $item-$currFrmNo]
					puts "C$item - [expr $item-$currFrmNo]"
					set currFrmNo $item
				}
			} else {

				# Add to repeated count
				incr frmRepCnt
			}

		} elseif {$item == 0 && $start != 0} {

			if {[expr ![string compare $outfmt "-Interlaced"]]} {
				if {$frmseq == 1 && $prevPicNo != 0} {
					set frmseq 0
				} else {
					incr frmDropCnt 
				}
			} else {
				# PIB Miss condition
				incr pibMissCnt
			}

		} elseif {$item != 0 && $start == 0} {
			# First valid frame with no.
			set start 1
			if {$item != 3} {
				incr frmDropCnt [expr $item-3]
			}
			set currFrmNo $item
			if {[expr ![string compare $outfmt "-Interlaced"]]} {
				set frmseq 1
			}
		} else {
		}

		set prevPicNo $item
	}

	# Format the result
	lappend result $frmDropCnt $frmRepCnt $pibMissCnt

	return $result
}

proc Bcdiff::bcCmpCRCVal {outfmt frmNolist1 crcvallist1 frmNolist2 crcvallist2} {

	set result {}
      set alredy 0 
	if {[expr ![string compare $outfmt "-Interlaced"]]} {
		for {set i 0} {$i < [llength $frmNolist1]} { } {
			set idx1 [lindex $frmNolist1 $i]
			if {$idx1 != 0} {

				set arridx1 [lsearch -exact $frmNolist1 $idx1] 
				set arridx2 [lsearch -exact $frmNolist2 $idx1]				
				if {$arridx2 != -1} {
					if {[lindex $crcvallist1 $arridx1] != [lindex $crcvallist2 $arridx2]} {
						lappend result [format "Field 1-%d: %.8X %.8X" $idx1 [lindex $crcvallist1 $arridx1] [lindex $crcvallist2 $arridx2]]
					}

					incr i

					set idx2 [lindex $frmNolist1 $i]
					if {$idx2 == 0 && [lindex $frmNolist2 [expr $arridx2+1]] == 0} {
						if {[lindex $crcvallist1 [expr $arridx1+1]] != [lindex $crcvallist2 [expr $arridx2+1]]} {
							  lappend result [format "Field 2-%d: %.8X %.8X" $idx1 [lindex $crcvallist1 [expr $arridx1+1]] [lindex $crcvallist2 [expr $arridx2+1]]]
						}
						incr i
					}
				} else {
					incr i
				}

			} else {
				incr i
			}
		}

	} else {
		foreach idx1 $frmNolist1 {
			if {$idx1 != 0} {
				set arridx1 [lsearch -exact $frmNolist1 $idx1] 
				set arridx2 [lsearch -exact $frmNolist2 $idx1]
                        
                        				
				if {$arridx2 != -1} {
					if {[lindex $crcvallist1 $arridx1] != [lindex $crcvallist2 $arridx2]} {
                                    if {$alredy != $idx1  } {
					  	    lappend result [format "%d: %.8X %.8X" $idx1 [lindex $crcvallist1 $arridx1] [lindex $crcvallist2 $arridx2]]
                                        set alredy  $idx1
                                    }
					}
				}
			}
		}
	}

	return $result
}

proc Bcdiff::bcOutputResult {file1 result1 file2 result2 cmpres} {
	
	# Print the header
	puts "\n\n---------------------------"
	puts "CRC File comparison result:"
	puts "---------------------------"

	puts "\nCRC File 1 : $file1"

	# Open the files
	set fh [open $file1 r]

	# Read the complete file and extract only the frame no and crc value list
	set filename [string trimleft [lindex [split [lindex [split [read $fh] "\n"] 1] ":"] 1]]
	puts "Input clip : $filename\n"

	close $fh

	puts "   Frame Drop Count     : [lindex $result1 0]"
	puts "   PIB Miss Count       : [lindex $result1 2]"
	puts "   Repeated Frame Count : [lindex $result1 1]"


	puts "\nCRC File 2 : $file2"

	# Open the files
	set fh [open $file2 r]

	# Read the complete file and extract only the frame no and crc value list
	set filename [string trimleft [lindex [split [lindex [split [read $fh] "\n"] 1] ":"] 1]]
	puts "Input clip : $filename\n"

	close $fh

	puts "   Frame Drop Count     : [lindex $result2 0]"
	puts "   PIB Miss Count       : [lindex $result2 2]"
	puts "   Repeated Frame Count : [lindex $result2 1]"

	puts "\nCRC Mis-match result:\n"
	puts "   Mis-match count      : [llength $cmpres]"
	puts "   Mis-match list (<Frame No.>: <CRC Value 1> <CRC Value 2>):\n"
      
	foreach item $cmpres {
		puts "\t$item"
	}

	return 0
}
